@@ -79,6 +79,18 @@ class CreateAppointment(Wizard):
     create_ = StateTransition()
     open_ = StateAction('health.action_gnuhealth_appointment_view')
 
+    @classmethod
+    def __setup__(cls):
+        super(CreateAppointment, cls).__setup__()
+        cls._error_messages.update({
+            'no_company_timezone':
+                'You need to define a timezone for this company'
+                ' before creating a work schedule.\n\n'
+                'Party > Configuration > Companies',
+            'end_before_start': 'End date before start',
+            'period_too_long': 'The schedule period should be < 31 days',
+        })
+
     def transition_create_(self):
         pool = Pool()
         Appointment = pool.get('gnuhealth.appointment')
@@ -90,10 +102,22 @@ class CreateAppointment(Wizard):
             company = Company(company_id)
             if company.timezone:
                 timezone = pytz.timezone(company.timezone)
+            else:
+                self.raise_user_error('no_company_timezone')
 
         appointments = []
+
         # Iterate over days
         day_count = (self.start.date_end - self.start.date_start).days + 1
+        
+        # Validate dates
+        if (self.start.date_start and self.start.date_end):
+            if (self.start.date_end < self.start.date_start):
+                self.raise_user_error('end_before_start')
+
+            if (day_count > 31):
+                self.raise_user_error('period_too_long')
+        
         for single_date in (self.start.date_start + timedelta(n)
             for n in range(day_count)):
             if ((single_date.weekday() == 0 and self.start.monday)
